"
    Copyright (C) 2016-now Jan Vrany

    This code is not an open-source (yet). You may use this code
    for your own experiments and projects, given that:

    * all modification to the code will be sent to the
      original author for inclusion in future releases
    * this is not used in any commercial software

    This license is provisional and may (will) change in
    a future.
"
"{ Package: 'jv:dragonfly/c1' }"

"{ NameSpace: DragonFly }"

Object subclass:#C1BasicBlock
	instanceVariableNames:'startPC endPC successors precedessors'
	classVariableNames:''
	poolDictionaries:'DragonFly::C1LLVMTypes'
	category:'DragonFly-C1'
!

!C1BasicBlock class methodsFor:'documentation'!

copyright
"
    Copyright (C) 2016-now Jan Vrany

    This code is not an open-source (yet). You may use this code
    for your own experiments and projects, given that:

    * all modification to the code will be sent to the
      original author for inclusion in future releases
    * this is not used in any commercial software

    This license is provisional and may (will) change in
    a future.

"
! !

!C1BasicBlock class methodsFor:'instance creation'!

new
    "return an initialized instance"

    ^ self basicNew initialize.
! !

!C1BasicBlock methodsFor:'accessing'!

endPC
    ^ endPC
!

endPC:anInteger
    self assert: startPC <= anInteger.
    endPC := anInteger.

    "Modified: / 15-08-2016 / 10:34:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

precedessors
    ^ precedessors
!

startPC
    ^ startPC
!

startPC:anInteger
    startPC := anInteger.
!

successors
    ^ successors
! !

!C1BasicBlock methodsFor:'adding & removing'!

addPrecedessor: aC1BasicBlock
    precedessors add: aC1BasicBlock

    "Created: / 15-08-2016 / 09:23:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

addSuccessor: aC1BasicBlock
    successors add: aC1BasicBlock.
    aC1BasicBlock addPrecedessor: self.

    "Created: / 15-08-2016 / 09:23:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1BasicBlock methodsFor:'enumerating'!

withAllPrecedessorsDo: aBlock
    "Evaluates `aBlock` with the receiver and all its
     precedessors recursively"

    | queue seen |

    queue := OrderedCollection with: self.
    seen := Set new.
    [ queue notEmpty ] whileTrue:[ 
        | block |

        block := queue removeFirst.
        (seen includes: block) ifFalse:[ 
            seen add: block.
            aBlock value: block.
            queue addAll: block precedessors.
        ].
    ].

    "Created: / 16-08-2016 / 14:38:53 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-08-2016 / 17:07:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

withAllSuccessorsDo: aBlock
    "Evaluates `aBlock` with the receiver and all its
     successors recursively"

    | queue seen |

    queue := OrderedCollection with: self.
    seen := Set new.
    [ queue notEmpty ] whileTrue:[ 
        | block |

        block := queue removeFirst.
        (seen includes: block) ifFalse:[ 
            seen add: block.
            aBlock value: block.
            queue addAll: (block successors asSortedCollection:[:a :b | a startPC < b startPC])  .
        ].
    ].

    "Created: / 16-08-2016 / 14:39:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 17-08-2016 / 22:24:19 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1BasicBlock methodsFor:'initialization'!

initialize
    "Invoked when a new instance is created."

    successors := Set new.
    precedessors := Set new.

    "/ super initialize.   -- commented since inherited method does nothing

    "Modified (comment): / 15-08-2016 / 09:24:26 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1BasicBlock methodsFor:'printing & storing'!

printOn:aStream
    "append a printed representation of the receiver to the argument, aStream"

    super printOn:aStream.
    aStream nextPutAll:'sPC: '.
    startPC printOn:aStream.
    aStream nextPutAll:' ePC: '.
    endPC printOn:aStream.

    "Modified: / 15-08-2016 / 10:37:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1BasicBlock methodsFor:'utilities'!

splitAtPC: pc
    "Splits receiver block into 2 blocks linked together such that
     first instruction in second block is at given PC."

    | splitted |

    splitted := C1BasicBlock new.
    splitted startPC: pc; endPC: endPC.
    endPC := pc - 1.
    successors do:[:each | splitted addSuccessor: each ].
    successors := Set new.
    self addSuccessor: splitted.  
    ^ splitted

    "Created: / 16-08-2016 / 17:06:31 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

