"{ Encoding: utf8 }"

"
    Copyright (C) 2016-now Jan Vrany

    This code is not an open-source (yet). You may use this code
    for your own experiments and projects, given that:

    * all modification to the code will be sent to the
      original author for inclusion in future releases
    * this is not used in any commercial software

    This license is provisional and may (will) change in
    a future.
"
"{ Package: 'jv:dragonfly/c1' }"

"{ NameSpace: DragonFly }"

JavaByteCodeInterpreter subclass:#C1CompilerJava
	instanceVariableNames:'backend labelMap blockMap'
	classVariableNames:''
	poolDictionaries:'DragonFly::C1CompilerDebugFlags'
	category:'DragonFly-C1-Java'
!

!C1CompilerJava class methodsFor:'documentation'!

copyright
"
    Copyright (C) 2016-now Jan Vrany

    This code is not an open-source (yet). You may use this code
    for your own experiments and projects, given that:

    * all modification to the code will be sent to the
      original author for inclusion in future releases
    * this is not used in any commercial software

    This license is provisional and may (will) change in
    a future.

"
! !

!C1CompilerJava class methodsFor:'initialization'!

initialize
    "We have to recompile some methods to make them bytecode methods
     to workaround STC bug, see issue #105

        https://swing.fit.cvut.cz/projects/stx-jv/ticket/105

    "
    self methodDictionary keysAndValuesDo:[ :selector :method |
        ((selector startsWith: 'if') and:[ method category = 'instructions' ]) ifTrue:[ 
            self recompile: selector.  
        ].
    ].

    "Created: / 22-08-2016 / 10:26:29 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1CompilerJava class methodsFor:'compilation'!

compile: aJavaMethod
    self new compile: aJavaMethod

    "Created: / 25-07-2016 / 19:00:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1CompilerJava methodsFor:'accessing'!

method: aJavaMethod
    self assert: aJavaMethod isJavaMethod.
    method := aJavaMethod

    "Created: / 24-07-2016 / 23:49:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1CompilerJava methodsFor:'compilation'!

compile    
    self assert: method isJavaMethod.

    backend := C1CompilerBackend new.
    backend method: method.  
    backend prepare.
    backend emitPrologue.

    method isAbstract ifTrue:[
        self compileAbstract 
    ] ifFalse:[
        method isNative ifTrue:[ 
            self compileNetive 
        ] ifFalse:[ 
            self compileBytecode
        ].
    ].
    backend emitEpilogue.
    backend finish.

    "Created: / 24-07-2016 / 23:52:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 19-08-2016 / 23:10:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

compile: aMethod
    self method: aMethod.
    self compile.

    "Created: / 30-07-2016 / 00:21:58 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1CompilerJava methodsFor:'compilation - private'!

compileBytecode
    self process: method

    "Created: / 31-07-2016 / 08:50:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 15-08-2016 / 23:12:04 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1CompilerJava methodsFor:'instructions'!

ifeq
    "
     if true, branch to instruction at branchoffset (signed short constructed from unsigned bytes branchbyte1 << 8 + branchbyte2)
     stack: value -> nothing
     args: branchByte1 branchByte2"
    
    | dest |

    dest := self fetchBytes2.
    self popI = 0 then: [ self relativeJump: dest ].

    "
     The value must be of type int. It is popped from the operand stack and
     compared against zero. All comparisons are signed. The results of the
     comparisons are as follows:
     eq succeeds if and only if value = 0
     ne succeeds if and only if value  0
     lt succeeds if and only if value < 0
     le succeeds if and only if value  0
     gt succeeds if and only if value > 0
     ge succeeds if and only if value  0
     If the comparison succeeds, the unsigned branchbyte1 and branchbyte2
     are used to construct a signed 16-bit offset, where the offset is
     calculated to be (branchbyte1 << 8) | branchbyte2. Execution then proceeds
     at that offset from the address of the opcode of this if<cond> instruction.
     The target address must be that of an opcode of an instruction within the
     method that contains this if<cond> instruction.
     Otherwise, execution proceeds at the address of the instruction following
     this if<cond> instruction."

    "Created: / 15-08-2016 / 23:32:24 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

ifge
    "
     if true, branch to instruction at branchoffset (signed short constructed from unsigned bytes branchbyte1 << 8 + branchbyte2)
     stack: value -> nothing
     args: branchByte1 branchByte2"
    
    | dest |

    dest := self fetchBytes2.
    self popI >= 0 then: [ self relativeJump: dest ].

    "
     The value must be of type int. It is popped from the operand stack and
     compared against zero. All comparisons are signed. The results of the
     comparisons are as follows:
     eq succeeds if and only if value = 0
     ne succeeds if and only if value  0
     lt succeeds if and only if value < 0
     le succeeds if and only if value  0
     gt succeeds if and only if value > 0
     ge succeeds if and only if value  0
     If the comparison succeeds, the unsigned branchbyte1 and branchbyte2
     are used to construct a signed 16-bit offset, where the offset is
     calculated to be (branchbyte1 << 8) | branchbyte2. Execution then proceeds
     at that offset from the address of the opcode of this if<cond> instruction.
     The target address must be that of an opcode of an instruction within the
     method that contains this if<cond> instruction.
     Otherwise, execution proceeds at the address of the instruction following
     this if<cond> instruction."

    "Created: / 15-08-2016 / 23:32:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

ifgt
    "
     if true, branch to instruction at branchoffset (signed short constructed from unsigned bytes branchbyte1 << 8 + branchbyte2)
     stack: value -> nothing
     args: branchByte1 branchByte2"
    
    | dest |

    dest := self fetchBytes2.
    self popI > 0 then: [ self relativeJump: dest ].

    "
     The value must be of type int. It is popped from the operand stack and
     compared against zero. All comparisons are signed. The results of the
     comparisons are as follows:
     eq succeeds if and only if value = 0
     ne succeeds if and only if value  0
     lt succeeds if and only if value < 0
     le succeeds if and only if value  0
     gt succeeds if and only if value > 0
     ge succeeds if and only if value  0
     If the comparison succeeds, the unsigned branchbyte1 and branchbyte2
     are used to construct a signed 16-bit offset, where the offset is
     calculated to be (branchbyte1 << 8) | branchbyte2. Execution then proceeds
     at that offset from the address of the opcode of this if<cond> instruction.
     The target address must be that of an opcode of an instruction within the
     method that contains this if<cond> instruction.
     Otherwise, execution proceeds at the address of the instruction following
     this if<cond> instruction."

    "Created: / 15-08-2016 / 23:32:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

ificmpeq
    "
     Branch if int comparison succeeds
     stack: value1 value2 -> nothing
     args: branchByte1 branchByte2"
    
    | dest |

    dest := self fetchBytes2.
    (self popI = self popI) then: [ self relativeJump: dest ].

    "
     Description
     Both value1 and value2 must be of type int. They are both popped from
     the operand stack and compared. All comparisons are signed. The results
     of the comparison are as follows:
     eq succeeds if and only if value1 = value2
     ne succeeds if and only if value1 ~= value2
     lt succeeds if and only if value1 < value2
     le succeeds if and only if value1  value2
     gt succeeds if and only if value1 > value2
     ge succeeds if and only if value1  value2
     If the comparison succeeds, the unsigned branchbyte1 and branchbyte2 are
     used to construct a signed 16-bit offset, where the offset is calculated
     to be (branchbyte1 << 8) | branchbyte2. Execution then proceeds at that
     offset from the address of the opcode of this if_icmp<cond> instruction.
     The target address must be that of an opcode of an instruction within the
     method that contains this if_icmp<cond> instruction.
     Otherwise, execution proceeds at the address of the instruction following
     this if_icmp<cond> instruction."

    "Created: / 15-08-2016 / 23:32:53 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

ificmpge
    "
     Branch if int comparison succeeds
     stack: value1 value2 -> nothing
     args: branchByte1 branchByte2"
    
    | dest |

    dest := self fetchBytes2.
    self swap.
    (self popI >= self popI) then: [ self relativeJump: dest ]

    "
     Description
     Both value1 and value2 must be of type int. They are both popped from
     the operand stack and compared. All comparisons are signed. The results
     of the comparison are as follows:
     eq succeeds if and only if value1 = value2
     ne succeeds if and only if value1 ~= value2
     lt succeeds if and only if value1 < value2
     le succeeds if and only if value1 <= value2
     gt succeeds if and only if value1 > value2
     ge succeeds if and only if value1 >= value2
     If the comparison succeeds, the unsigned branchbyte1 and branchbyte2 are
     used to construct a signed 16-bit offset, where the offset is calculated
     to be (branchbyte1 << 8) | branchbyte2. Execution then proceeds at that
     offset from the address of the opcode of this if_icmp<cond> instruction.
     The target address must be that of an opcode of an instruction within the
     method that contains this if_icmp<cond> instruction.
     Otherwise, execution proceeds at the address of the instruction following
     this if_icmp<cond> instruction."

    "Created: / 15-08-2016 / 23:32:58 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

ificmpgt
    "
     Branch if int comparison succeeds
     stack: value1 value2 -> nothing
     args: branchByte1 branchByte2"
    
    | dest |

    dest := self fetchBytes2.
    self swap.
    (self popI > self popI) then: [ self relativeJump: dest ]

    "
     Description
     Both value1 and value2 must be of type int. They are both popped from
     the operand stack and compared. All comparisons are signed. The results
     of the comparison are as follows:
     eq succeeds if and only if value1 = value2
     ne succeeds if and only if value1 ~= value2
     lt succeeds if and only if value1 < value2
     le succeeds if and only if value1 <= value2
     gt succeeds if and only if value1 > value2
     ge succeeds if and only if value1 >= value2
     If the comparison succeeds, the unsigned branchbyte1 and branchbyte2 are
     used to construct a signed 16-bit offset, where the offset is calculated
     to be (branchbyte1 << 8) | branchbyte2. Execution then proceeds at that
     offset from the address of the opcode of this if_icmp<cond> instruction.
     The target address must be that of an opcode of an instruction within the
     method that contains this if_icmp<cond> instruction.
     Otherwise, execution proceeds at the address of the instruction following
     this if_icmp<cond> instruction."

    "Created: / 15-08-2016 / 23:33:04 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

ificmple
    "
     Branch if int comparison succeeds
     stack: value1 value2 -> nothing
     args: branchByte1 branchByte2"
    
    | dest |

    dest := self fetchBytes2.
    self swap.
    (self popI <= self popI) then: [ self relativeJump: dest ]

    "
     Description
     Both value1 and value2 must be of type int. They are both popped from
     the operand stack and compared. All comparisons are signed. The results
     of the comparison are as follows:
     eq succeeds if and only if value1 = value2
     ne succeeds if and only if value1 ~= value2
     lt succeeds if and only if value1 < value2
     le succeeds if and only if value1 <= value2
     gt succeeds if and only if value1 > value2
     ge succeeds if and only if value1 >= value2
     If the comparison succeeds, the unsigned branchbyte1 and branchbyte2 are
     used to construct a signed 16-bit offset, where the offset is calculated
     to be (branchbyte1 << 8) | branchbyte2. Execution then proceeds at that
     offset from the address of the opcode of this if_icmp<cond> instruction.
     The target address must be that of an opcode of an instruction within the
     method that contains this if_icmp<cond> instruction.
     Otherwise, execution proceeds at the address of the instruction following
     this if_icmp<cond> instruction."

    "Created: / 15-08-2016 / 23:33:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

ificmplt
    "
     Branch if int comparison succeeds
     stack: value1 value2 -> nothing
     args: branchByte1 branchByte2"
    
    | dest |

    dest := self fetchBytes2.
    self swap.
    (self popI < self popI) then: [ self relativeJump: dest ]

    "
     Description
     Both value1 and value2 must be of type int. They are both popped from
     the operand stack and compared. All comparisons are signed. The results
     of the comparison are as follows:
     eq succeeds if and only if value1 = value2
     ne succeeds if and only if value1 ~= value2
     lt succeeds if and only if value1 < value2
     le succeeds if and only if value1 <= value2
     gt succeeds if and only if value1 > value2
     ge succeeds if and only if value1 >= value2
     If the comparison succeeds, the unsigned branchbyte1 and branchbyte2 are
     used to construct a signed 16-bit offset, where the offset is calculated
     to be (branchbyte1 << 8) | branchbyte2. Execution then proceeds at that
     offset from the address of the opcode of this if_icmp<cond> instruction.
     The target address must be that of an opcode of an instruction within the
     method that contains this if_icmp<cond> instruction.
     Otherwise, execution proceeds at the address of the instruction following
     this if_icmp<cond> instruction."

    "Created: / 15-08-2016 / 23:33:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

ificmpne
    "
     Branch if int comparison succeeds
     stack: value1 value2 -> nothing
     args: branchByte1 branchByte2"
    
    | dest |

    dest := self fetchBytes2.
    (self popI ~= self popI) then: [ self relativeJump: dest ].

    "
     Description
     Both value1 and value2 must be of type int. They are both popped from
     the operand stack and compared. All comparisons are signed. The results
     of the comparison are as follows:
     eq succeeds if and only if value1 = value2
     ne succeeds if and only if value1 ~= value2
     lt succeeds if and only if value1 < value2
     le succeeds if and only if value1  value2
     gt succeeds if and only if value1 > value2
     ge succeeds if and only if value1  value2
     If the comparison succeeds, the unsigned branchbyte1 and branchbyte2 are
     used to construct a signed 16-bit offset, where the offset is calculated
     to be (branchbyte1 << 8) | branchbyte2. Execution then proceeds at that
     offset from the address of the opcode of this if_icmp<cond> instruction.
     The target address must be that of an opcode of an instruction within the
     method that contains this if_icmp<cond> instruction.
     Otherwise, execution proceeds at the address of the instruction following
     this if_icmp<cond> instruction."

    "Created: / 15-08-2016 / 23:33:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

ifle
    "
     if true, branch to instruction at branchoffset (signed short constructed from unsigned bytes branchbyte1 << 8 + branchbyte2)
     stack: value -> nothing
     args: branchByte1 branchByte2"
    
    | dest |

    dest := self fetchBytes2.
    self popI <= 0 then: [ self relativeJump: dest ].

    "
     The value must be of type int. It is popped from the operand stack and
     compared against zero. All comparisons are signed. The results of the
     comparisons are as follows:
     eq succeeds if and only if value = 0
     ne succeeds if and only if value  0
     lt succeeds if and only if value < 0
     le succeeds if and only if value  0
     gt succeeds if and only if value > 0
     ge succeeds if and only if value  0
     If the comparison succeeds, the unsigned branchbyte1 and branchbyte2
     are used to construct a signed 16-bit offset, where the offset is
     calculated to be (branchbyte1 << 8) | branchbyte2. Execution then proceeds
     at that offset from the address of the opcode of this if<cond> instruction.
     The target address must be that of an opcode of an instruction within the
     method that contains this if<cond> instruction.
     Otherwise, execution proceeds at the address of the instruction following
     this if<cond> instruction."

    "Created: / 15-08-2016 / 23:33:30 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

iflt
    "
     if true, branch to instruction at branchoffset (signed short constructed from unsigned bytes branchbyte1 << 8 + branchbyte2)
     stack: value -> nothing
     args: branchByte1 branchByte2"
    
    | dest |

    dest := self fetchBytes2.
    self popI < 0 then: [ self relativeJump: dest ].

    "
     The value must be of type int. It is popped from the operand stack and
     compared against zero. All comparisons are signed. The results of the
     comparisons are as follows:
     eq succeeds if and only if value = 0
     ne succeeds if and only if value  0
     lt succeeds if and only if value < 0
     le succeeds if and only if value  0
     gt succeeds if and only if value > 0
     ge succeeds if and only if value  0
     If the comparison succeeds, the unsigned branchbyte1 and branchbyte2
     are used to construct a signed 16-bit offset, where the offset is
     calculated to be (branchbyte1 << 8) | branchbyte2. Execution then proceeds
     at that offset from the address of the opcode of this if<cond> instruction.
     The target address must be that of an opcode of an instruction within the
     method that contains this if<cond> instruction.
     Otherwise, execution proceeds at the address of the instruction following
     this if<cond> instruction."

    "Created: / 15-08-2016 / 23:33:34 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

ifne
    "
     if value is not 0, branch to instruction at branchoffset (signed short constructed from unsigned bytes branchbyte1 << 8 + branchbyte2)
     stack: value -> nothing
     args: branchByte1 branchByte2"
    "/ -1 in block - because we already increased pc to point to the next instruction
    
    | dest |

    dest := self fetchBytes2.
    self popI ~= 0 then: [ self relativeJump: dest ].

    "
     The value must be of type int. It is popped from the operand stack and
     compared against zero. All comparisons are signed. The results of the
     comparisons are as follows:
     eq succeeds if and only if value = 0
     ne succeeds if and only if value ~= 0
     lt succeeds if and only if value < 0
     le succeeds if and only if value <= 0
     gt succeeds if and only if value > 0
     ge succeeds if and only if value >= 0
     If the comparison succeeds, the unsigned branchbyte1 and branchbyte2
     are used to construct a signed 16-bit offset, where the offset is
     calculated to be (branchbyte1 << 8) | branchbyte2. Execution then proceeds
     at that offset from the address of the opcode of this if<cond> instruction.
     The target address must be that of an opcode of an instruction within the
     method that contains this if<cond> instruction.
     Otherwise, execution proceeds at the address of the instruction following
     this if<cond> instruction."

    "Created: / 15-08-2016 / 23:33:56 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

ifnonnull
    | value target |

    value := self popI.
    target := self fetchIndex2.
    value notNil then:[
        self relativeJump: target.
    ]
    "
    ifnonnull

    Operation

       Branch if reference not null

    Format

       ifnonnull
       branchbyte1
       branchbyte2

    Forms

       ifnonnull = 199 (0xc7)

    Operand Stack

       ..., value →

       ...

    Description

       The value must be of type reference. It is popped from the
       operand stack. If value is not null, the unsigned branchbyte1
       and branchbyte2 are used to construct a signed 16-bit offset,
       where the offset is calculated to be (branchbyte1 << 8) |
       branchbyte2. Execution then proceeds at that offset from the
       address of the opcode of this ifnonnull instruction. The target
       address must be that of an opcode of an instruction within the
       method that contains this ifnonnull instruction.

       Otherwise, execution proceeds at the address of the instruction
       following this ifnonnull instruction.
    "

    "Created: / 15-08-2016 / 23:34:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1CompilerJava methodsFor:'private-push / pop'!

popI
    | v |

    v := super popI.
    self assert: v isC1Int32Value.
    ^ v

    "Created: / 03-08-2016 / 19:48:26 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 06-08-2016 / 18:09:51 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

push: aC1Value
    sp := sp + 1.
    aC1Value setBackend: backend.
    context at: sp put: aC1Value

    "Created: / 03-08-2016 / 19:38:41 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 10-08-2016 / 22:24:38 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

pushI:aValueOrInteger
    | v |

    v := aValueOrInteger isInteger 
            ifTrue:[ C1Value int32: aValueOrInteger ]
            ifFalse:[ aValueOrInteger ].
    self assert: v isC1Int32Value.
    self push: v.

    "Created: / 31-07-2016 / 23:05:04 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 11-08-2016 / 09:04:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

pushJ:aValueOrInteger
    | v |

    v := aValueOrInteger isInteger 
            ifTrue:[ C1Value int64: aValueOrInteger ]
            ifFalse:[ aValueOrInteger ].
    self assert: v isInt64.
    sp := sp + 1.
    context at: sp put: v.

    "Created: / 31-07-2016 / 23:05:26 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 03-08-2016 / 11:57:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

relativeJump: delta 
    | targetPC |

    targetPC := instrPointer + delta.
    delta < 0 ifTrue:[ 
        backend emitInterruptCheck.
    ].
    backend emitJumpTo: (labelMap at: (blockMap at: targetPC))

    "Created: / 15-08-2016 / 23:46:54 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 22-08-2016 / 23:51:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1CompilerJava methodsFor:'processing loop'!

enterProcessingLoop
    self shouldNotImplement.

    "Created: / 09-08-2016 / 09:21:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 15-08-2016 / 23:09:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

enterProcessingLoopFor: aC1BasicBlock
    pc := aC1BasicBlock startPC.
    wide := false.
    backend placeLabel: (labelMap at: aC1BasicBlock).  
    [ pc <= aC1BasicBlock endPC ] whileTrue:[
        | op |

        op := byteCode at: pc.
        pc := pc + 1.
        self switch: op    
    ].

    "Created: / 15-08-2016 / 22:14:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

leaveProcessorWith: aC1Value
    backend emitReturn: aC1Value objValue

    "Created: / 04-08-2016 / 08:57:35 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 06-08-2016 / 09:28:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

process: aMethod
    | entry |

    method := aMethod.
    byteCode := aMethod byteCode.
    constantPool := aMethod constantPool.
    pc := 1.
    wide := false.
    leaveProcessor := false.     
    context := C1Context forMethod: aMethod.
    method localVariableTable notNil ifTrue:[ 
        method method localVariableTable do:[:entry | 
            (entry startPC == 0 and:[ entry endPC == (method byteCode size - 1) ]) ifTrue:[
                backend diLine: (method lineNumberForPC: entry startPC).
                backend diDeclareVariable: entry name at: entry slot.
            ].
        ].
    ].     
    sp := aMethod numArgs + aMethod numVars + (aMethod isStatic ifTrue:[ 0 ] ifFalse:[ 1 ]).
    entry := C1BasicBlockAnalyzerJava process: aMethod.
    labelMap := Dictionary new.
    blockMap := Dictionary new.
    entry withAllSuccessorsDo:[ :block |
        blockMap at: block startPC put: block.
        labelMap at: block put: (backend createLabel: 'pc', block startPC printString).
    ].
    entry withAllSuccessorsDo:[ :block | 
        self enterProcessingLoopFor: block. 
    ].

    "Created: / 15-08-2016 / 22:13:12 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 22-09-2016 / 23:45:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

switch: op
    backend diLine: (method lineNumberForPC:pc - 1).
    super switch: op.

    "Created: / 23-08-2016 / 22:44:06 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1CompilerJava class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !


C1CompilerJava initialize!
