"
    Copyright (C) 2016-now Jan Vrany

    This code is not an open-source (yet). You may use this code
    for your own experiments and projects, given that:

    * all modification to the code will be sent to the
      original author for inclusion in future releases
    * this is not used in any commercial software

    This license is provisional and may (will) change in
    a future.
"
"{ Package: 'jv:dragonfly/c1' }"

"{ NameSpace: DragonFly }"

Object subclass:#C1Value
	instanceVariableNames:'objValue context spilled backend'
	classVariableNames:''
	poolDictionaries:'DragonFly::C1LLVMTypes'
	category:'DragonFly-C1'
!

!C1Value class methodsFor:'documentation'!

copyright
"
    Copyright (C) 2016-now Jan Vrany

    This code is not an open-source (yet). You may use this code
    for your own experiments and projects, given that:

    * all modification to the code will be sent to the
      original author for inclusion in future releases
    * this is not used in any commercial software

    This license is provisional and may (will) change in
    a future.

"
! !

!C1Value class methodsFor:'instance creation'!

bool: anLLVMValueOrBoolean
    ^ C1BoolValue new numValue: anLLVMValueOrBoolean

    "Created: / 19-08-2016 / 23:29:59 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

int32: anLLVMValueOrSmallInteger
    ^ C1Int32Value new numValue: anLLVMValueOrSmallInteger

    "Created: / 03-08-2016 / 12:01:19 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 05-08-2016 / 19:34:03 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

int32obj: anLLVMValueOrSmallInteger
    | v |

    anLLVMValueOrSmallInteger isInteger ifTrue:[ 
        v := LLVMConstant sint32: (anLLVMValueOrSmallInteger << 1) | 1
    ] ifFalse:[ 
        v := anLLVMValueOrSmallInteger.
    ].
    ^ Int32Obj new setValue: v.

    "Created: / 03-08-2016 / 19:20:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 05-08-2016 / 16:58:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

new
    "return an initialized instance"

    ^ self basicNew initialize.
! !

!C1Value class methodsFor:'testing'!

isAbstract
    ^ self == DragonFly::C1Value
! !

!C1Value methodsFor:'accessing'!

index
    ^ context identityIndexOf: self.

    "Modified: / 17-08-2016 / 22:39:41 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

objType
    ^ TyOBJ

    "Created: / 06-08-2016 / 07:32:12 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

objValue
    "Return LLVMValue representing an Smalltalk object"
    
    objValue isNil ifTrue:[
        | index |

        index := self index.
        self assert:index notNil description:'No context index set!!'.
        self assert:spilled description:'Value not spilled and not in LLVM register!!'.
        objValue := backend loadContextVarAt:index.
    ].
    ^ objValue.

    "Created: / 05-08-2016 / 18:39:41 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 10-08-2016 / 22:23:39 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

objValue: anLLVMValue
    self assert: anLLVMValue isLLVMValue.
    self assert: anLLVMValue type = self objType.
    objValue := anLLVMValue

    "Created: / 05-08-2016 / 19:37:13 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 06-08-2016 / 07:32:50 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1Value methodsFor:'converting'!

asC1DoubleValue
    self error: 'Value of this kind cannt be converted to requested value'

    "Created: / 17-08-2016 / 22:30:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

asC1FloatValue
    self error: 'Value of this kind cannt be converted to requested value'

    "Created: / 17-08-2016 / 22:30:19 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

asC1Int32Value
    self error: 'Value of this kind cannt be converted to requested value'

    "Created: / 17-08-2016 / 22:30:24 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

asC1Int64Value
    self error: 'Value of this kind cannt be converted to requested value'

    "Created: / 17-08-2016 / 22:30:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

asC1ReferenceValue
    self error: 'Value of this kind cannt be converted to requested value'

    "Created: / 17-08-2016 / 22:30:40 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

asC1SmallIntegerValue
    self error: 'Value of this kind cannt be converted to requested value'

    "Created: / 17-08-2016 / 22:30:34 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1Value methodsFor:'initialization'!

initialize
    "Invoked when a new instance is created."

    spilled := false.

    "Modified: / 03-08-2016 / 19:18:38 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 05-08-2016 / 18:38:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

setBackend: aC1Backend
    self assert: (backend isNil or:[backend == aC1Backend]) description: 'Trying to set backend twice!!'.
    backend := aC1Backend

    "Created: / 03-08-2016 / 18:44:51 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 10-08-2016 / 22:48:53 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

setContext: aC1Context
    self assert: (context isNil or:[context == aC1Context]).
    context := aC1Context

    "Created: / 10-08-2016 / 22:25:29 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

setSpilled: aBoolean
    spilled := aBoolean

    "Created: / 05-08-2016 / 18:38:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1Value methodsFor:'printing & storing'!

printOn:aStream
    "append a printed representation of the receiver to the argument, aStream"

    super printOn:aStream.
    aStream nextPut:$(.
    aStream nextPutAll: (spilled ifTrue:[ 'S ' ] ifFalse:[ '  ' ]).
    aStream nextPutAll:'OV='.
    objValue printOn:aStream.
    aStream nextPut:$).

    "Modified: / 15-08-2016 / 13:42:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1Value methodsFor:'testing'!

isC1DoubleValue
    ^ false

    "Created: / 06-08-2016 / 18:04:40 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isC1FloatValue
    ^ false

    "Created: / 06-08-2016 / 18:04:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isC1Int32Value
    ^ false

    "Created: / 06-08-2016 / 18:04:54 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isC1Int64Value
    ^ false

    "Created: / 06-08-2016 / 18:05:03 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isC1ReferenceValue
    ^ false

    "Created: / 06-08-2016 / 18:05:19 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isC1SmallIntegerValue
    ^ false

    "Created: / 06-08-2016 / 18:05:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

