"
    Copyright (C) 2016-now Jan Vrany

    This code is not an open-source (yet). You may use this code
    for your own experiments and projects, given that:

    * all modification to the code will be sent to the
      original author for inclusion in future releases
    * this is not used in any commercial software

    This license is provisional and may (will) change in
    a future.
"
"{ Package: 'jv:dragonfly/c1/tests' }"

"{ NameSpace: DragonFly }"

TestCase subclass:#C1CompilerBackendTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:'DragonFly::C1LLVMTypes LLVMIntPredicate VMData VMOffsets'
	category:'DragonFly-C1-Tests'
!

!C1CompilerBackendTests class methodsFor:'documentation'!

copyright
"
    Copyright (C) 2016-now Jan Vrany

    This code is not an open-source (yet). You may use this code
    for your own experiments and projects, given that:

    * all modification to the code will be sent to the
      original author for inclusion in future releases
    * this is not used in any commercial software

    This license is provisional and may (will) change in
    a future.

"
! !

!C1CompilerBackendTests methodsFor:'helpers'!

returnSenderContext
    ^ thisContext sender.

    "Created: / 09-07-2016 / 08:00:08 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

sum: a and: b
    ^ a + b

    "Created: / 23-06-2016 / 22:21:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1CompilerBackendTests methodsFor:'tests'!

test_context_vars_01
    | method  compiler ret |

    method := Method new.
    method numberOfArgs:1.
    method numberOfVars:0.
    method stackSize:0.
    compiler := C1CompilerBackend new.
    compiler method:method.
    compiler prepare.
    compiler emitPrologue.
    compiler emitReturn:(compiler loadContextVarAt: 1).
    compiler emitEpilogue.
    compiler finish.
    self assert:method code notNil.
    ret := method 
            valueWithReceiver:self
            arguments:#(1)
            selector:testSelector.
    self assert:ret == 1.

    "Created: / 31-07-2016 / 23:14:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_thisContext_01
    | method  compiler  ctx |

    method := Method new.
    method numberOfArgs:0.
    method numberOfVars:0.
    method stackSize:0.
    compiler := C1CompilerBackend new.
    compiler method:method.
    compiler prepare.
    compiler emitPrologue.
    compiler emitReturn:(compiler 
                emitSend:#returnSenderContext
                to:(compiler loadLiteral:self)
                with:#()).
    compiler emitEpilogue.
    compiler finish.
    self assert:method code notNil.
    ctx := method 
            valueWithReceiver:self
            arguments:#()
            selector:testSelector.
    self assert:ctx isContext.
    self assert:ctx receiver == self.
    self assert:ctx selector == testSelector.
    self assert:ctx method == method.
    self assert:ctx numArgs == 0.
    self assert:ctx numVars == 0.
    self assert:ctx numTemps == 0.

    "Created: / 24-06-2016 / 00:11:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 25-07-2016 / 19:01:27 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_thisContext_02
    | method  compiler  ctx |

    method := JavaMethod new.
    method numberOfArgs:0.
    method numberOfVars:0.
    method stackSize:0.
    method setAccessFlags: JavaConstants ACC_PUBLIC.
    compiler := C1CompilerBackend new.
    compiler method:method.
    compiler prepare.
    compiler emitPrologue.
    compiler emitReturn:(compiler 
                emitSend:#returnSenderContext
                to:(compiler loadLiteral:self)
                with:#()).
    compiler emitEpilogue.
    compiler finish.
    self assert:method code notNil.
    ctx := method 
            valueWithReceiver:self
            arguments:#()
            selector:testSelector.
    self assert:ctx isJavaContext.
    self assert:ctx receiver == self.
    self assert:ctx selector == testSelector.
    self assert:ctx method == method.
    self assert:ctx numArgs == 0.
    self assert:ctx numVars == 0.
    self assert:ctx numTemps == 0.

    "Created: / 07-08-2016 / 21:53:31 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1CompilerBackendTests methodsFor:'tests - building blocks'!

test_emitInterruptCheck
    | cookie  method  compiler  asm |

    cookie := Object new.
    method := Method new.
    method numberOfArgs:0.
    method numberOfVars:0.
    method stackSize:0.
    compiler := C1CompilerBackend new.
    compiler method:method.
    compiler prepare.
    compiler emitInterruptCheck.
    asm := compiler instVarNamed: #asm.
    asm ret: compiler loadNil.
    compiler finish.
    self assert:method code notNil.
    self assert:(method valueWithReceiver:nil arguments:#()) == cookie.

    "Created: / 22-08-2016 / 23:44:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_emitReturn_01
    | method  compiler |

    method := Method new.
    method numberOfArgs:0.
    method numberOfVars:0.
    method stackSize:0.
    compiler := C1CompilerBackend new.
    compiler method:method.
    compiler prepare.
    compiler emitPrologue.
    compiler emitReturn:(compiler loadLiteral:123).
    compiler emitEpilogue.
    compiler finish.
    self assert:method code notNil.
    self assert:(method valueWithReceiver:nil arguments:#()) == 123

    "Created: / 23-06-2016 / 22:35:12 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 25-07-2016 / 19:01:03 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_emitSend_01
    | method  compiler  asm |

    method := Method new.
    method numberOfArgs:0.
    method numberOfVars:0.
    method stackSize:0.
    compiler := C1CompilerBackend new.
    compiler method:method.
    compiler prepare.
    asm := compiler instVarNamed:#asm.
    asm ret:(compiler 
                emitSend:#sum:and:
                to:(compiler loadLiteral:self)
                with:{
                        compiler loadLiteral:1.
                        compiler loadLiteral:2
                    }).
    compiler finish.
    self assert:method code notNil.
    self assert:(method valueWithReceiver:nil arguments:#()) == 3

    "Created: / 23-06-2016 / 22:21:19 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 25-07-2016 / 19:01:06 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_emitSend_02
    | method  compiler  asm |

    method := Method new.
    method numberOfArgs:0.
    method numberOfVars:0.
    method stackSize:0.
    compiler := C1CompilerBackend new.
    compiler method:method.
    compiler prepare.
    asm := compiler instVarNamed:#asm.
    asm ret:(compiler 
                emitSend:#yourself
                to:(compiler loadLiteral:self)
                with:#()).
    compiler finish.
    self assert:method code notNil.
    self assert:(method valueWithReceiver:nil arguments:#()) == self

    "Created: / 23-06-2016 / 22:28:37 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 25-07-2016 / 19:01:08 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_isSmallIntegerObject
    | compiler  module  function  asm  jit  test |

    compiler := C1CompilerBackend new.
    module := LLVMModule newWithName:testSelector.
    function := module addFunctionNamed:'test'
            type:(LLVMType function:{ TyOBJ } returning:LLVMType int1).
    asm := function builder.
    compiler
        instVarNamed:#module put:module;
        instVarNamed:#function put:function;
        instVarNamed:#asm put:asm.
    asm ret:(compiler emitIsSmallIntegerObject:(function parameterAt:1)).
    jit := LLVMExecutionEngine newForModule:module.
    test := jit externalOfFunction:function.
    self assert:(test callWith:1) == 1.
    self assert:(test callWith:self) == 0.

    "Created: / 12-02-2016 / 12:17:57 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 25-07-2016 / 19:01:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_loadClassOf
    | method  compiler  asm  obj |

    method := Method new.
    method numberOfArgs:2.
    method numberOfVars:0.
    method stackSize:0.
    compiler := C1CompilerBackend new.
    compiler method:method.
    compiler prepare.
    asm := compiler instVarNamed:#asm.
    obj := (compiler instVarNamed:#function) parameterAt:OBJFUNCArgIndexArg1.
    asm ret:(compiler loadClassOf:obj).
    compiler finish.
    self assert:method code notNil.
    self 
        assert:(method valueWithReceiver:nil arguments:{ 1.0 . 2 }) == Double.
    self 
        assert:(method valueWithReceiver:nil arguments:{ self . 2 }) == self class.
    self 
        assert:(method valueWithReceiver:nil arguments:{ 1 . 2 }) == SmallInteger

    "Created: / 12-02-2016 / 12:27:26 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 25-07-2016 / 19:01:14 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_loadClassSmallInteger
    | method  compiler  asm |

    method := Method new.
    method numberOfArgs:2.
    method numberOfVars:0.
    method stackSize:0.
    compiler := C1CompilerBackend new.
    compiler method:method.
    compiler prepare.
    asm := compiler instVarNamed:#asm.
    asm ret:(compiler loadClassSmallInteger).
    compiler finish.
    self assert:method code notNil.
    self 
        assert:(method valueWithReceiver:nil arguments:#( 1 2 )) == SmallInteger

    "Created: / 12-02-2016 / 12:26:14 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 25-07-2016 / 19:01:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_loadLiteral
    | cookie  method  compiler  asm |

    cookie := Object new.
    method := Method new.
    method numberOfArgs:0.
    method numberOfVars:0.
    method stackSize:0.
    compiler := C1CompilerBackend new.
    compiler method:method.
    compiler prepare.
    asm := compiler instVarNamed:#asm.
    asm ret:(compiler loadLiteral:cookie).
    compiler finish.
    self assert:method code notNil.
    self assert:(method valueWithReceiver:nil arguments:#()) == cookie.

    "Created: / 20-06-2016 / 23:40:58 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 25-07-2016 / 19:01:18 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1CompilerBackendTests methodsFor:'tests - codegen'!

test_codegen
    | method  compiler  asm |

    method := Method new.
    method numberOfArgs:2.
    method numberOfVars:0.
    method stackSize:0.
    compiler := C1CompilerBackend new.
    compiler method:method.
    compiler prepare.
    asm := compiler instVarNamed:#asm.
    asm ret:(asm int:(LLVMConstant uintptr:0) toPtr:TyOBJ).
    compiler finish.
    self assert:method code notNil.
    self assert:(method valueWithReceiver:nil arguments:#( 1 2 )) == nil

    "Created: / 21-04-2016 / 13:48:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 25-07-2016 / 19:01:22 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1CompilerBackendTests methodsFor:'tests - prologue / epilogue'!

test_prologue_epilogue_01
    | method  compiler |

    method := Method new.
    method numberOfArgs:0.
    method numberOfVars:0.
    method stackSize:0.
    compiler := C1CompilerBackend new.
    compiler method:method.
    compiler prepare.
    compiler emitPrologue.
    compiler storeRetvalTemp:compiler loadNil.
    compiler emitEpilogue.
    compiler finish.
    self assert:method code notNil.
    self assert:(method valueWithReceiver:nil arguments:#()) isNil

    "Created: / 17-06-2016 / 17:39:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 25-07-2016 / 19:01:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!C1CompilerBackendTests class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

